#!/usr/bin/env python
# created by chris@drumminhands.com
# see instructions at http://www.drumminhands.com/2014/06/15/raspberry-pi-photo-booth/

import os
import glob
import time
from time import sleep
import RPi.GPIO as GPIO
import picamera  # http://picamera.readthedocs.org/en/release-1.4/install2.html
import atexit
import pygame
import sys
from pygame.locals import QUIT, KEYDOWN, K_ESCAPE
import config  # this is the config python file config.py


####################
# Variables Config #
####################
led_pin = 7  # LED pin for the start button
flash_pin = 11  # Pin for the LED camera flash
btn_pin = 18  # pin for the start button

total_pics = 3  # number of pics to be taken
capture_delay = 1  # delay between pics
prep_delay = 5  # number of seconds at step 1 as users prep to have photo taken
gif_delay = 100  # How much time between frames in the animated gif
restart_delay = 5  # how long to display finished message before beginning a new session

# full frame of v1 camera is 2592x1944. Wide screen max is 2592,1555
# if you run into resource issues, try smaller, like 1920x1152. 
# or increase memory http://picamera.readthedocs.io/en/release-1.12/fov.html#hardware-limits
high_res_w = 3200  # width of high res image, if taken
high_res_h = 1920  # height of high res image, if taken

#########################
# Variables that Change #
#########################
# Do not change these variables, as the code will change it anyway
transform_x = config.monitor_w  # how wide to scale the jpg when replaying
transform_y = config.monitor_h  # how high to scale the jpg when replaying
offset_x = 0  # how far off to left corner to display photos
offset_y = 0  # how far off to left corner to display photos
replay_delay = 1  # how much to wait in-between showing pics on-screen after taking
replay_cycles = 1  # how many times to show each photo on-screen after taking

################
# Other Config #
################
real_path = os.path.dirname(os.path.realpath(__file__))

# GPIO setup
GPIO.setmode(GPIO.BOARD)
GPIO.setup(led_pin, GPIO.OUT)  # LED button
GPIO.setup(flash_pin, GPIO.OUT)  # LED flash
GPIO.setup(btn_pin, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.output(led_pin, False)  # for some reason the pin turns on at the beginning of the program.
GPIO.output(flash_pin, False)

# initialize pygame
pygame.init()
pygame.display.set_mode((config.monitor_w, config.monitor_h))
screen = pygame.display.get_surface()
pygame.display.set_caption('Photo Booth Pics')
pygame.mouse.set_visible(False)  # hide the mouse cursor
pygame.display.toggle_fullscreen()


#############
# Functions #
#############

# clean up running programs as needed when main program exits
def cleanup():
    print('Ended abruptly')
    pygame.quit()
    GPIO.cleanup()


atexit.register(cleanup)


# A function to handle keyboard/mouse/device input events
def input_event_handler(events):
    for event in events:  # Hit the ESC key to quit the slideshow.
        if (event.type == QUIT or
                (event.type == KEYDOWN and event.key == K_ESCAPE)):
            pygame.quit()
            GPIO.cleanup()


# delete files in folder
def clear_pics(channel):
    files = glob.glob(config.file_path + '*')
    for f in files:
        os.remove(f)
    # light the lights in series to show completed
    print("Deleted previous pics")
    for i in range(0, 3):  # blink light
        GPIO.output(led_pin, True)
        sleep(0.25)
        GPIO.output(led_pin, False)
        sleep(0.25)


# set variables to properly display the image on screen at right ratio
def set_dimensions(img_w, img_h):
    # Note this only works when in booting in desktop mode.
    # When running in terminal, the size is not correct (it displays small). Why?

    # connect to global vars
    global transform_y, transform_x, offset_y, offset_x

    # based on output screen resolution, calculate how to display
    ratio_h = (config.monitor_w * img_h) / img_w

    if ratio_h < config.monitor_h:
        # Use horizontal black bars
        # print "horizontal black bars"
        transform_y = ratio_h
        transform_x = config.monitor_w
        offset_y = (config.monitor_h - ratio_h) / 2
        offset_x = 0
    elif ratio_h > config.monitor_h:
        # Use vertical black bars
        # print "vertical black bars"
        transform_x = (config.monitor_h * img_w) / img_h
        transform_y = config.monitor_h
        offset_x = (config.monitor_w - transform_x) / 2
        offset_y = 0
    else:
        # No need for black bars as photo ratio equals screen ratio
        # print "no black bars"
        transform_x = config.monitor_w
        transform_y = config.monitor_h
        offset_y = offset_x = 0

        # uncomment these lines to troubleshoot screen ratios


#     print str(img_w) + " x " + str(img_h)
#     print "ratio_h: "+ str(ratio_h)
#     print "transform_x: "+ str(transform_x)
#     print "transform_y: "+ str(transform_y)
#     print "offset_y: "+ str(offset_y)
#     print "offset_x: "+ str(offset_x)

# display one image on screen
def show_image(image_path):
    # clear the screen
    screen.fill((0, 0, 0))

    # load the image
    img = pygame.image.load(image_path)
    img = img.convert()

    # set pixel dimensions based on image
    set_dimensions(img.get_width(), img.get_height())

    # rescale the image to fit the current display
    img = pygame.transform.scale(img, (transform_x, transform_y))
    screen.blit(img, (offset_x, offset_y))
    pygame.display.flip()


# display a blank screen
def clear_screen():
    screen.fill((0, 0, 0))
    pygame.display.flip()


# display a group of images
def display_pics(jpg_group):
    for i in range(0, replay_cycles):  # show pics a few times
        for j in range(1, total_pics + 1):  # show each pic
            show_image(config.file_path + jpg_group + "-0" + str(j) + ".jpg")
            time.sleep(replay_delay)  # pause


# define the photo taking function for when the big button is pressed 
def start_photobooth():
    input_event_handler(pygame.event.get())  # press escape to exit pygame. Then press ctrl-c to exit python.

    # ################################ Begin Step 1 #################################

    print("Get Ready")
    GPIO.output(led_pin, False)
    show_image(real_path + "/instructions.png")
    sleep(prep_delay)

    # clear the screen
    clear_screen()

    with picamera.PiCamera() as camera:
        camera.vflip = False
        camera.hflip = True  # flip for preview, showing users a mirror image
        # camera.saturation = -100 # comment out this line if you want color images
        camera.iso = config.camera_iso

        camera.resolution = (high_res_w, high_res_h)  # set camera resolution to high res

        # ############################ Begin Step 2 #################################

        print("Taking pics")

        now = time.strftime("%Y-%m-%d-%H-%M-%S")  # get the current date and time for the start of the filename

        if config.capture_count_pics:
            try:  # take the photos
                for i in range(1, total_pics + 1):
                    camera.hflip = False  # preview a mirror image
                    if config.flash_enabled:
                        GPIO.output(flash_pin, True)
                    camera.start_preview(
                        resolution=(config.monitor_w, config.monitor_h))  # start preview at low res but the right ratio
                    time.sleep(2)  # warm up camera
                    GPIO.output(led_pin, True)  # turn on the LED
                    filename = config.file_path + now + '-0' + str(i) + '.jpg'
                    camera.hflip = False  # flip back when taking photo
                    camera.capture(filename)
                    GPIO.output(flash_pin, False)
                    print(filename)
                    GPIO.output(led_pin, False)  # turn off the LED
                    camera.stop_preview()
                    show_image(real_path + "/pose" + str(i) + ".png")
                    time.sleep(capture_delay)  # pause in-between shots
                    clear_screen()
                    if i == total_pics + 1:
                        break
            finally:
                camera.close()
                GPIO.output(flash_pin, False)
        else:
            camera.start_preview(
                resolution=(config.monitor_w, config.monitor_h))  # start preview at low res but the right ratio
            time.sleep(2)  # warm up camera

            try:  # take the photos
                for i, filename in enumerate(
                        camera.capture_continuous(config.file_path + now + '-' + '{counter:02d}.jpg')):
                    GPIO.output(led_pin, True)  # turn on the LED
                    print(filename)
                    time.sleep(capture_delay)  # pause in-between shots
                    GPIO.output(led_pin, False)  # turn off the LED
                    if i == total_pics - 1:
                        break
            finally:
                camera.stop_preview()
                camera.close()

    # ########################## Begin Step 3 #################################

    input_event_handler(pygame.event.get())  # press escape to exit pygame. Then press ctrl-c to exit python.

    show_image(real_path + "/processing.png")

    # ########################## Begin Step 4 #################################

    input_event_handler(pygame.event.get())  # press escape to exit pygame. Then press ctrl-c to exit python.

    try:
        display_pics(now)
    except ValueError:
        pygame.quit()
        sys.exit()

    print("Done")

    show_image(real_path + "/finished2.png")

    time.sleep(restart_delay)
    show_image(real_path + "/intro.png")
    GPIO.output(led_pin, True)  # turn on the LED


################
# Main Program #
################

# clear the previously stored pics based on config settings
if config.clear_on_startup:
    clear_pics(1)

print("Photo booth app running...")
for x in range(0, 5):  # blink light to show the app is running
    GPIO.output(led_pin, True)
    sleep(0.25)
    GPIO.output(led_pin, False)
    sleep(0.25)

show_image(real_path + "/intro.png")

while True:
    GPIO.output(led_pin, True)  # turn on the light showing users they can push the button
    input_event_handler(pygame.event.get())  # press escape to exit pygame. Then press ctrl-c to exit python.
    GPIO.wait_for_edge(btn_pin, GPIO.FALLING)
    time.sleep(config.debounce)  # debounce
    start_photobooth()
